import ddf.minim.*;
import ddf.minim.signals.*;
import ddf.minim.analysis.*;
import ddf.minim.effects.*;

PFont font;
PFont fontBig;
String logText = null;
boolean introPlayed = false;
int introTime = 5000;
double introTimeMillis;
String[] allScores;

Minim minim;
AudioSample playerGold;
AudioSample playerHealth;
AudioSample playerHurt;
AudioSample playerBonus;
AudioSample playerStep;

PImage bkg1;
PImage bkg2;
PImage bkg3;
PImage frg;
PImage playerI;
Tile[][] tileA;
Player player;
boolean showMap = false;
boolean allTilesObserved = false;
boolean gameEnded = false;
boolean showScoreList = false;
boolean newScoreAdded = false;

int cellW;
int cellH;
int tileW;
int tileH;

boolean buttonDownUP = false;
boolean buttonDownDOWN = false;
boolean buttonDownLEFT = false;
boolean buttonDownRIGHT = false;
boolean wasButtonDownUP = false;
boolean wasButtonDownDOWN = false;
boolean wasButtonDownRIGHT = false;
boolean wasButtonDownLEFT = false;
int reactionTimeMax = 3000;
int reactionTime = reactionTimeMax;
double millisReactionTime;

void setup(){
  frameRate(10);
  screen.width = 640;
  screen.height = 640;
  size(screen.width, screen.height);      

  minim = new Minim(this);
  playerGold = minim.loadSample("gold.wav", 1024);
  playerHealth = minim.loadSample("health.wav", 1024);
  playerHurt = minim.loadSample("foe.wav", 1024);
  playerBonus = minim.loadSample("bonus.wav", 1024);
  playerStep = minim.loadSample("step.wav", 1024);

  font = loadFont("KellyAnnGothic-32.vlw");
  fontBig = loadFont("KellyAnnGothic-50.vlw");  
  allScores = loadStrings("scores.txt");

  bkg1 = loadImage("grass.png");
  bkg2 = loadImage("hills.png");
  bkg3 = loadImage("mountains.png");
  frg = loadImage("foreground.png");

  playerI = loadImage("scout.png");  
  resetGame();   
}

void draw(){
  if(!introPlayed){
    drawIntro();
  }else if(showScoreList){
    drawScoreList();
  }else if(player.getHealth() <= 0){
    drawLooseGame();
  }
  else if(allTilesObserved){
    drawEndGame(player);
  }
  else{
    reactionTime = reactionTime - (int)(millis() - millisReactionTime);
    boolean playerMoved = false;
    if(buttonDownUP || wasButtonDownUP){
      player.setY(player.getY()-1);
      playerMoved = true;
      wasButtonDownUP = false;
    }
    if(buttonDownDOWN || wasButtonDownDOWN){    
      player.setY(player.getY()+1);
      playerMoved = true;
      wasButtonDownDOWN = false;
    }
    if(buttonDownLEFT || wasButtonDownLEFT){
      player.setX(player.getX()-1);
      playerMoved = true;
      wasButtonDownLEFT = false;
    }
    if(buttonDownRIGHT || wasButtonDownRIGHT){
      player.setX(player.getX()+1);
      playerMoved = true;
      wasButtonDownRIGHT = false;
    }
    if(playerMoved){
      this.getPlayerStep().trigger();
    }

    allTilesObserved = true;
    for(int i = 0; i < cellW; ++i){
      for(int j = 0; j < cellH; ++j){
        if(player.getX() == i && player.getY() == j
          || player.getX() == i-1 && player.getY() == j-1
          || player.getX() == i-1 && player.getY() == j
          || player.getX() == i-1 && player.getY() == j+1
          || player.getX() == i && player.getY() == j-1
          || player.getX() == i && player.getY() == j+1
          || player.getX() == i+1 && player.getY() == j-1
          || player.getX() == i+1 && player.getY() == j
          || player.getX() == i+1 && player.getY() == j+1      
          ){
          tileA[i][j].setObservable(true);
        }

        image(tileA[i][j].getBkg(), i*tileW, j*tileH);

        PImage img = null;
        img = tileA[i][j].getBonusImage();
        if(img != null){
          image(tileA[i][j].getBonusImage(), i*tileW, j*tileH);
        }
        img = tileA[i][j].getExtraImage();
        if(img != null){
          image(tileA[i][j].getExtraImage(), i*tileW, j*tileH);
        }      
        img = tileA[i][j].getFoeImage();
        if(img != null){
          image(tileA[i][j].getFoeImage(), i*tileW, j*tileH);
        }

        if(!tileA[i][j].isObservable() && !showMap){
          image(frg, i*tileW, j*tileH);
        }

        if((int)player.getX() == i && (int)player.getY() == j){            
          image(player.getImage(), player.getX()*tileW, player.getY()*tileH);          
          tileA[i][j].fireActions(this, player);            
        }

        if(tileA[i][j].isObservable() != true){
          allTilesObserved = false;
        }
      }
    }

    textFont(font, 32);
    if(logText != null){      
      text(logText, 0, 32);
    }
    text("health", screen.width-100, 32);
    fill(player.getHealthColor());
    rect(screen.width - 8, 5, 5, player.getHealth());
    fill(color(255, 255, 255));

  }
}

void keyPressed(){
  if(keyCode == LEFT){
    buttonDownLEFT = true;
    wasButtonDownLEFT = true;
  }
  if(keyCode == RIGHT){
    buttonDownRIGHT = true;
    wasButtonDownRIGHT = true;
  }
  if(keyCode == UP){
    buttonDownUP = true;
    wasButtonDownUP = true;
  }
  if(keyCode == DOWN){
    buttonDownDOWN = true;
    wasButtonDownDOWN = true;
  }
}

void keyReleased() {
  if(keyCode == LEFT){
    buttonDownLEFT = false;
  }
  if(keyCode == RIGHT){
    buttonDownRIGHT = false;
  }
  if(keyCode == UP){
    buttonDownUP = false;
  }
  if(keyCode == DOWN){
    buttonDownDOWN = false;
  }
  if((keyCode == ENTER || keyCode == RETURN) && gameEnded){
    gameEnded = false;
    resetGame();
  }
  if(keyCode == 32 && gameEnded){    
    showScoreList = true;
  }
  reactionTime = reactionTimeMax;
  millisReactionTime = millis();
}

public void setLogText(String logText){
  this.logText = logText;
}

private void drawIntro(){
  for(int i = 0; i < cellW; ++i){
      for(int j = 0; j < cellH; ++j){
        image(bkg1, i*tileW, j*tileH);
      }
  }
  textFont(fontBig, 50);
  text("Scouting only!", 0, screen.height/2);
  textFont(font, 32);
  text("You are the scout of Civilization.\nMove with arrow keys to explore the area.\nBeware of foes!", 0, screen.height/2 + 50);
  if(millis() - introTimeMillis < introTime){
    introPlayed = false;
  }else{
    introPlayed = true;
  }
}

private void drawScoreList(){
  for(int i = 0; i < cellW; ++i){
      for(int j = 0; j < cellH; ++j){
        image(bkg1, i*tileW, j*tileH);
      }
  }
  textFont(fontBig, 50);
  text("Biggest gold amount", 0, 50);
    
  String allScoresEdited = "";
  if(allScores == null){
    allScores = new String[1];
  }  
  for(int i = 0; i < allScores.length; ++i){    
    if(allScores[i] != null && player.getGold() > Integer.valueOf(allScores[i]) && !newScoreAdded){      
      allScores[i] = Integer.toString(player.getGold());
      newScoreAdded = true;
    }else if(allScores[i] == null && !newScoreAdded){      
      allScores[i] = Integer.toString(player.getGold());
      newScoreAdded = true;
    }
    allScoresEdited = allScoresEdited + allScores[i] + "\n";
  }
  text(allScoresEdited, 0, 100);
  if(millis() - introTimeMillis < introTime){
    introPlayed = false;
  }else{
    introPlayed = true;
  }
}

private void drawLooseGame(){
  textFont(fontBig, 50);
  text("Game over!\nPress enter for new game.", 0, screen.height/2);
  gameEnded = true;
}

private void drawEndGame(Player p){
  textFont(fontBig, 50);
  if(!gameEnded){  
    p.setGold(p.getGold() + p.getHealth());
  }
  text("You collected " + p.getGold() + " gold!", 0, screen.height/2);
  textFont(font, 32);
  text("Press enter for new game.\nPress space to see biggest gold amount.", 0, screen.height/2 + 50);
  gameEnded = true;
}

private void resetGame(){
  allTilesObserved = false;
  gameEnded = false;
  newScoreAdded = false;
  showScoreList = false;
  logText = null;
  reactionTime = 0;
  cellW = screen.width/bkg1.width;
  cellH = screen.height/bkg1.height;
  tileW = bkg1.width;
  tileH = bkg1.height;

  player = new Player(playerI, cellW - 1, cellH - 1);
  player.setX(cellW - 1);
  player.setY(cellH - 1);

  tileA = new Tile[cellW][cellH];
  int bkg3a = cellW*cellH/6;  
  int bkg2a = cellW*cellH/4; 
  int foea =  cellW*cellH/20;
  int extraa =  cellW*cellH/10;
  int bonusa =  cellW*cellH/3;

  for(int i = 0; i < cellW; ++i){
    for(int j = 0; j < cellH; ++j){      
      int k = (int)random(1, 4);
      if(k == 1){
        tileA[i][j] = new Tile(this, bkg1);
      }
      else if(k == 2 && bkg2a > 0){
        tileA[i][j] = new Tile(this, bkg2);
        --bkg2a;
      }
      else if(k == 3 && bkg3a > 0){
        tileA[i][j] = new Tile(this, bkg3);
        --bkg3a;
      }
      else{
        tileA[i][j] = new Tile(this, bkg1);
      }

      int extraBonusOrFoe = (int)random(0,5);
      if(extraBonusOrFoe == 0){
        k = (int)random(0,3);
        if(k > 0 && extraa > 0){
          tileA[i][j].setExtra(k);
          --extraa;
        }
      }
      else if(extraBonusOrFoe == 1){
        k = (int)random(0,6);
        if(k > 0 && extraa > 0){
          tileA[i][j].setBonus(k);
          --bonusa;
        }
      }
      else if(extraBonusOrFoe == 2){
        k = (int)random(0,5);
        if(k > 0 && extraa > 0){
          tileA[i][j].setFoe(k);
          --foea;
        }
      }

      if(player.getX() == i && player.getY() == j){
        tileA[i][j].setVisited(true);        
      }
    }
  }
}

void stop()
{
  saveStrings("scores.txt", allScores);
  playerGold.close();
  minim.stop();
  super.stop();
}

public AudioSample getPlayerGold(){
  return this.playerGold;
}
public AudioSample getPlayerHealth(){
  return this.playerHealth;
}
public AudioSample getPlayerHurt(){
  return this.playerHurt;
}
public AudioSample getPlayerBonus(){
  return this.playerBonus;
}
public AudioSample getPlayerStep(){
  return this.playerStep;
}








